@@ -12,14 +12,16 @@ use crate::font_introspector::text::Script;
 use crate::font_introspector::Metrics;
 use crate::layout::render_data::RenderData;
 use crate::layout::RichTextLayout;
+use crate::Graphics;
 use lru::LruCache;
 use rustc_hash::FxHashMap;
+use std::collections::HashSet;
 use std::num::NonZeroUsize;
 use std::sync::atomic::{AtomicUsize, Ordering};
 
 use crate::font_introspector::Attributes;
 use crate::font_introspector::Setting;
-use crate::{sugarloaf::primitives::SugarCursor, Graphic};
+use crate::{sugarloaf::primitives::SugarCursor, DrawableChar, Graphic};
 
 pub struct RichTextCounter(AtomicUsize);
 
@@ -33,34 +35,46 @@ impl RichTextCounter {
     }
 }
 
-/// Data that describes a fragment.
 #[derive(Debug, Clone)]
 pub struct FragmentData {
     pub content: String,
-    /// Style
     pub style: FragmentStyle,
 }
 
-#[derive(Default, Clone)]
+#[derive(Default, Clone, Debug)]
 pub struct BuilderLine {
-    /// Collection of fragments.
     pub fragments: Vec<FragmentData>,
     pub render_data: RenderData,
 }
 
-/// Builder state.
+#[derive(Default, Clone, PartialEq)]
+#[repr(C)]
+pub enum BuilderStateUpdate {
+    #[default]
+    Full,
+    Partial(HashSet<usize>),
+    Noop,
+}
+
 #[derive(Default)]
 pub struct BuilderState {
-    /// Lines State
     pub lines: Vec<BuilderLine>,
-    /// Font variation setting cache.
     pub vars: FontSettingCache<f32>,
+    pub last_update: BuilderStateUpdate,
     metrics_cache: MetricsCache,
     scaled_font_size: f32,
     pub layout: RichTextLayout,
 }
 
 impl BuilderState {
+    #[inline]
+    pub fn new_line_at(&mut self, pos: usize) {
+        self.lines.insert(pos, BuilderLine::default());
+    }
+    #[inline]
+    pub fn remove_line_at(&mut self, pos: usize) {
+        self.lines.remove(pos);
+    }
     #[inline]
     pub fn from_layout(layout: &RichTextLayout) -> Self {
         Self {
@@ -78,9 +92,32 @@ impl BuilderState {
         self.lines.len().wrapping_sub(1)
     }
     #[inline]
+    pub fn mark_clean(&mut self) {
+        self.last_update = BuilderStateUpdate::Noop;
+    }
+    #[inline]
+    pub fn mark_dirty(&mut self) {
+        self.last_update = BuilderStateUpdate::Full;
+    }
+    #[inline]
+    pub fn mark_line_dirty(&mut self, line: usize) {
+        match &mut self.last_update {
+            BuilderStateUpdate::Full => {
+                // No operation
+            }
+            BuilderStateUpdate::Noop => {
+                self.last_update = BuilderStateUpdate::Partial(HashSet::from([line]));
+            }
+            BuilderStateUpdate::Partial(set) => {
+                set.insert(line);
+            }
+        };
+    }
+    #[inline]
     pub fn clear(&mut self) {
         self.lines.clear();
         self.vars.clear();
+        self.last_update = BuilderStateUpdate::Full;
     }
     #[inline]
     pub fn rescale(&mut self, scale_factor: f32) {
@@ -102,10 +139,11 @@ impl BuilderState {
         if prev_font_size != self.scaled_font_size {
             self.metrics_cache.inner.clear();
         }
+        self.last_update = BuilderStateUpdate::Full;
     }
 
     pub fn increase_font_size(&mut self) -> bool {
-        if self.layout.font_size < 40.0 {
+        if self.layout.font_size < 100.0 {
             self.layout.font_size += 1.0;
             self.update_font_size();
             return true;
@@ -232,6 +270,8 @@ pub struct FragmentStyle {
     pub cursor: Option<SugarCursor>,
     /// Media
     pub media: Option<Graphic>,
+    /// Drawable character
+    pub drawable_char: Option<DrawableChar>,
 }
 
 impl Default for FragmentStyle {
@@ -250,6 +290,7 @@ impl Default for FragmentStyle {
             decoration: None,
             decoration_color: None,
             media: None,
+            drawable_char: None,
         }
     }
 }
@@ -331,6 +372,13 @@ impl Content {
         self.states.remove(rich_text_id);
     }
 
+    #[inline]
+    pub fn mark_states_clean(&mut self) {
+        for state in self.states.values_mut() {
+            state.mark_clean();
+        }
+    }
+
     #[inline]
     pub fn update_dimensions(
         &mut self,
@@ -347,7 +395,11 @@ impl Content {
                 .build();
             let render_data = content.get_state(&id).unwrap().lines[0].clone();
 
-            if let Some(dimension) = advance_brush.dimensions(&self.fonts, &render_data) {
+            if let Some(dimension) = advance_brush.dimensions(
+                &self.fonts,
+                &render_data,
+                &mut Graphics::default(),
+            ) {
                 rte.layout.dimensions.height = dimension.height;
                 rte.layout.dimensions.width = dimension.width;
             }
@@ -380,6 +432,28 @@ impl Content {
         self
     }
 
+    #[inline]
+    pub fn new_line_at(&mut self, pos: usize) -> &mut Content {
+        if let Some(selector) = self.selector {
+            if let Some(content) = self.states.get_mut(&selector) {
+                content.new_line_at(pos);
+            }
+        }
+
+        self
+    }
+
+    #[inline]
+    pub fn remove_line_at(&mut self, pos: usize) -> &mut Content {
+        if let Some(selector) = self.selector {
+            if let Some(content) = self.states.get_mut(&selector) {
+                content.remove_line_at(pos);
+            }
+        }
+
+        self
+    }
+
     #[inline]
     pub fn clear_line(&mut self, line_to_clear: usize) -> &mut Content {
         if let Some(selector) = self.selector {
@@ -439,6 +513,7 @@ impl Content {
     ) -> &mut Content {
         if let Some(selector) = self.selector {
             if let Some(state) = self.states.get_mut(&selector) {
+                state.mark_line_dirty(line);
                 let line = &mut state.lines[line];
 
                 line.fragments.push(FragmentData {
@@ -471,137 +546,124 @@ impl Content {
         self
     }
 
-    #[inline]
-    pub fn build(&mut self) {
-        if let Some(selector) = self.selector {
-            if let Some(state) = self.states.get_mut(&selector) {
-                let script = Script::Latin;
-                for line_number in 0..state.lines.len() {
-                    let line = &mut state.lines[line_number];
-                    for item in &line.fragments {
-                        let vars = state.vars.get(item.style.font_vars);
-                        let shaper_key = &item.content;
+    // Helper function to process a single line that avoids borrow issues
+    fn process_line(&mut self, state_id: usize, line_number: usize) {
+        // Get all needed data while borrowing parts of self separately
+        let script = Script::Latin;
 
-                        // println!("{:?} -> {:?}", item.style.font_id, shaper_key);
+        // Safe to get state first as we'll only use it to access properties
+        let state = match self.states.get_mut(&state_id) {
+            Some(state) => state,
+            None => return,
+        };
 
-                        if let Some(shaper) =
-                            self.word_cache.get(&item.style.font_id, shaper_key)
-                        {
-                            if let Some(metrics) =
-                                state.metrics_cache.inner.get(&item.style.font_id)
-                            {
-                                if line.render_data.push_run_without_shaper(
-                                    item.style,
-                                    state.scaled_font_size,
-                                    line_number as u32,
-                                    shaper,
-                                    metrics,
-                                ) {
-                                    continue;
-                                }
-                            }
-                        }
+        // Get references to the scaled font size and features outside any other borrows
+        let scaled_font_size = state.scaled_font_size;
+        let features = &self.font_features;
 
-                        self.word_cache.font_id = item.style.font_id;
-                        self.word_cache.content = item.content.clone();
-                        let font_library = { &mut self.fonts.inner.lock() };
-                        if let Some(data) = font_library.get_data(&item.style.font_id) {
-                            let mut shaper = self
-                                .scx
-                                .builder(data)
-                                .script(script)
-                                .size(state.scaled_font_size)
-                                .features(self.font_features.iter().copied())
-                                .variations(vars.iter().copied())
-                                .build();
+        // Check if the line exists
+        if line_number >= state.lines.len() {
+            return;
+        }
 
-                            shaper.add_str(&self.word_cache.content);
+        // Process fragments in the line
+        let line = &mut state.lines[line_number];
 
-                            state
-                                .metrics_cache
-                                .inner
-                                .entry(item.style.font_id)
-                                .or_insert_with(|| shaper.metrics());
+        // Process each fragment
+        for fragment_idx in 0..line.fragments.len() {
+            // Get a reference to the current fragment
+            let item = &line.fragments[fragment_idx];
+            let font_id = item.style.font_id;
+            let font_vars = item.style.font_vars;
+            let content = &item.content;
+            let style = item.style;
 
-                            line.render_data.push_run(
-                                item.style,
-                                state.scaled_font_size,
-                                line_number as u32,
-                                shaper,
-                                &mut self.word_cache,
-                            );
-                        }
+            // Get vars for this fragment
+            let vars: Vec<_> = state.vars.get(font_vars).to_vec();
+
+            // Check if the shaped text is already in the cache
+            if let Some(cache_entry) = self.word_cache.get(&font_id, content) {
+                if let Some(metrics) = state.metrics_cache.inner.get(&font_id) {
+                    if line.render_data.push_run_without_shaper(
+                        style,
+                        scaled_font_size,
+                        line_number as u32,
+                        cache_entry,
+                        metrics,
+                    ) {
+                        continue;
                     }
                 }
             }
+
+            // If not in cache, shape the text
+            // Set up cache entry info
+            self.word_cache.font_id = font_id;
+            self.word_cache.content = content.clone();
+
+            // Process the font data directly without cloning FontRef
+            {
+                let font_library = &mut self.fonts.inner.lock();
+                if let Some(data) = font_library.get_data(&font_id) {
+                    let mut shaper = self
+                        .scx
+                        .builder(data) // Use reference directly without cloning
+                        .script(script)
+                        .size(scaled_font_size)
+                        .features(features.iter().copied())
+                        .variations(vars.iter().copied())
+                        .build();
+
+                    shaper.add_str(content);
+
+                    // Cache metrics if needed
+                    state
+                        .metrics_cache
+                        .inner
+                        .entry(font_id)
+                        .or_insert_with(|| shaper.metrics());
+
+                    // Push run to render data
+                    line.render_data.push_run(
+                        style,
+                        scaled_font_size,
+                        line_number as u32,
+                        shaper,
+                        &mut self.word_cache,
+                    );
+                }
+            }
         }
     }
 
+    #[inline]
+    pub fn build(&mut self) {
+        // let start = std::time::Instant::now();
+        if let Some(selector) = self.selector {
+            let state_id = selector;
+
+            if let Some(state) = self.states.get_mut(&state_id) {
+                state.mark_dirty();
+                for line_number in 0..state.lines.len() {
+                    self.process_line(state_id, line_number);
+                }
+            }
+        }
+
+        // let duration = start.elapsed();
+        // println!("Time elapsed in build() is: {:?}", duration);
+    }
+
     #[inline]
     pub fn build_line(&mut self, line_number: usize) {
         if let Some(selector) = self.selector {
-            if let Some(state) = self.states.get_mut(&selector) {
-                let script = Script::Latin;
-                let line = &mut state.lines[line_number];
-                for item in &line.fragments {
-                    let vars = state.vars.get(item.style.font_vars);
-                    let shaper_key = &item.content;
-
-                    // println!("{:?} -> {:?}", item.style.font_id, shaper_key);
-
-                    if let Some(shaper) =
-                        self.word_cache.get(&item.style.font_id, shaper_key)
-                    {
-                        if let Some(metrics) =
-                            state.metrics_cache.inner.get(&item.style.font_id)
-                        {
-                            if line.render_data.push_run_without_shaper(
-                                item.style,
-                                state.scaled_font_size,
-                                line_number as u32,
-                                shaper,
-                                metrics,
-                            ) {
-                                continue;
-                            }
-                        }
-                    }
-
-                    self.word_cache.font_id = item.style.font_id;
-                    self.word_cache.content = item.content.clone();
-                    let font_library = { &mut self.fonts.inner.lock() };
-                    if let Some(data) = font_library.get_data(&item.style.font_id) {
-                        let mut shaper = self
-                            .scx
-                            .builder(data)
-                            .script(script)
-                            .size(state.scaled_font_size)
-                            .features(self.font_features.iter().copied())
-                            .variations(vars.iter().copied())
-                            .build();
-
-                        shaper.add_str(&self.word_cache.content);
-
-                        state
-                            .metrics_cache
-                            .inner
-                            .entry(item.style.font_id)
-                            .or_insert_with(|| shaper.metrics());
-
-                        line.render_data.push_run(
-                            item.style,
-                            state.scaled_font_size,
-                            line_number as u32,
-                            shaper,
-                            &mut self.word_cache,
-                        );
-                    }
-                }
-            }
+            // Process just the specified line
+            self.process_line(selector, line_number);
         }
     }
 }
 
+#[derive(Default)]
 pub struct WordCache {
     pub inner: FxHashMap<usize, LruCache<String, Vec<OwnedGlyphCluster>>>,
     stash: Vec<OwnedGlyphCluster>,
@@ -623,7 +685,7 @@ impl WordCache {
     pub fn get(
         &mut self,
         font_id: &usize,
-        content: &String,
+        content: &str,
     ) -> Option<&Vec<OwnedGlyphCluster>> {
         if let Some(cache) = self.inner.get_mut(font_id) {
             return cache.get(content);
@@ -640,7 +702,6 @@ impl WordCache {
     pub fn finish(&mut self) {
         if !self.content.is_empty() && !self.stash.is_empty() {
             if let Some(cache) = self.inner.get_mut(&self.font_id) {
-                // println!("{:?} {:?}", self.content, cache.len());
                 cache.put(
                     std::mem::take(&mut self.content),
                     std::mem::take(&mut self.stash),
