@@ -11,7 +11,6 @@ pub use wgpu;
 pub use font_introspector::{Stretch, Style, Weight};
 
 pub use crate::sugarloaf::{
-    compositors::SugarCompositors,
     graphics::{
         ColorType, Graphic, GraphicData, GraphicId, Graphics, ResizeCommand,
         ResizeParameter, MAX_GRAPHIC_DIMENSIONS,
@@ -20,8 +19,8 @@ pub use crate::sugarloaf::{
     Sugarloaf, SugarloafErrors, SugarloafRenderer, SugarloafWindow, SugarloafWindowSize,
     SugarloafWithErrors,
 };
-pub use components::quad::{ComposedQuad, Quad};
-pub use components::rect::Rect;
+pub use components::filters::Filter;
+pub use components::quad::Quad;
 pub use layout::{
     Content, FragmentStyle, FragmentStyleDecoration, SugarDimensions, UnderlineInfo,
     UnderlineShape,
